﻿using BlacksmithWorkshopContracts.BindingModels;
using BlacksmithWorkshopContracts.SearchModels;
using BlacksmithWorkshopContracts.StoragesContracts;
using BlacksmithWorkshopContracts.ViewModels;
using BlacksmithWorkshopDatabaseImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BlacksmithWorkshopDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public List<OrderViewModel> GetFullList()
        {
            using var context = new BlacksmithWorkshopDataBase();
            return context.Orders
            .Select(x => AccessManufactureStorage(x.GetViewModel))
            .ToList();
        }
        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return new();
            }
            using var context = new BlacksmithWorkshopDataBase();
            return context.Orders
            .Where(x => x.Id == model.Id)
            .Select(x => AccessManufactureStorage(x.GetViewModel))
            .ToList();
        }
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new BlacksmithWorkshopDataBase();
            return AccessManufactureStorage(context.Orders.FirstOrDefault(x => x.Id == model.Id)?.GetViewModel);
        }
        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            using var context = new BlacksmithWorkshopDataBase();
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return AccessManufactureStorage(newOrder.GetViewModel);
        }
        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new BlacksmithWorkshopDataBase();
            var order = context.Orders.FirstOrDefault(x => x.Id ==
            model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
            return AccessManufactureStorage(order.GetViewModel);
        }
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new BlacksmithWorkshopDataBase();
            var element = context.Orders.FirstOrDefault(rec => rec.Id ==
            model.Id);
            if (element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();
                return AccessManufactureStorage(element.GetViewModel);
            }
            return null;
        }

        public static OrderViewModel AccessManufactureStorage(OrderViewModel model)
        {
            if (model == null)
                return null;
            using var context = new BlacksmithWorkshopDataBase();
            foreach (var manufacture in context.Manufactures)
            {
                if (manufacture.Id == model.ManufactureId)
                {
                    model.ManufactureName = manufacture.ManufactureName;
                    break;
                }
            }
            return model;
        }
    }
}
